@@ -26,7 +26,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-preferences-dialog.c 6379 2008-08-10 16:40:21Z pborelli $
+ * $Id$
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,6 +48,7 @@
 #include "gedit-style-scheme-manager.h"
 #include "gedit-plugin-manager.h"
 #include "gedit-help.h"
+#include "gedit-dirs.h"
 
 /*
  * gedit-preferences dialog is a singleton since we don't
@@ -916,7 +917,7 @@ uninstall_scheme_clicked (GtkButton              *button,
 			if (new_iter_set)
 				gtk_tree_model_get (model, &new_iter,
 						    ID_COLUMN, &new_id,
-						    NULL);
+						    -1);
 						    			
 			real_new_id = populate_color_scheme_list (dlg, new_id);
 			g_free (new_id);
@@ -1062,14 +1063,16 @@ gedit_preferences_dialog_init (GeditPreferencesDialog *dlg)
 {
 	GtkWidget *error_widget;
 	gboolean ret;
+	gchar *file;
 	gchar *root_objects[] = {
 		"notebook",
 		"adjustment1",
 		"adjustment2",
 		"adjustment3",
+		"install_scheme_image",
 		NULL
 	};
-	
+
 	gedit_debug (DEBUG_PREFS);
 
 	dlg->priv = GEDIT_PREFERENCES_DIALOG_GET_PRIVATE (dlg);
@@ -1097,7 +1100,8 @@ gedit_preferences_dialog_init (GeditPreferencesDialog *dlg)
 			  G_CALLBACK (dialog_response_handler),
 			  NULL);
 	
-	ret = gedit_utils_get_ui_objects (GEDIT_UIDIR "gedit-preferences-dialog.ui",
+	file = gedit_dirs_get_ui_file ("gedit-preferences-dialog.ui");
+	ret = gedit_utils_get_ui_objects (file,
 		root_objects,
 		&error_widget,
 
@@ -1135,6 +1139,7 @@ gedit_preferences_dialog_init (GeditPreferencesDialog *dlg)
 		"plugin_manager_place_holder", &dlg->priv->plugin_manager_place_holder,
 
 		NULL);
+	g_free (file);
 
 	if (!ret)
 	{
