@@ -63,6 +63,9 @@
 #ifdef G_OS_WIN32
 #define SAVE_DATADIR DATADIR
 #undef DATADIR
+#include <io.h>
+#include <conio.h>
+#define _WIN32_WINNT 0x0500
 #include <windows.h>
 #define DATADIR SAVE_DATADIR
 #undef SAVE_DATADIR
@@ -79,7 +82,6 @@ static BaconMessageConnection *connection;
 /* command line */
 static gint line_position = 0;
 static gchar *encoding_charset = NULL;
-static const GeditEncoding *encoding;
 static gboolean new_window_option = FALSE;
 static gboolean new_document_option = FALSE;
 static gchar **remaining_args = NULL;
@@ -132,6 +134,24 @@ static const GOptionEntry options [] =
 	{NULL}
 };
 
+static void
+free_command_line_data (void)
+{
+	g_slist_foreach (file_list, (GFunc) g_object_unref, NULL);
+	g_slist_free (file_list);
+	file_list = NULL;
+
+	g_strfreev (remaining_args);
+	remaining_args = NULL;
+
+	g_free (encoding_charset);
+	encoding_charset = NULL;
+	
+	new_window_option = FALSE;
+	new_document_option = FALSE;
+	line_position = 0;
+}
+
 static void
 gedit_get_command_line_data (void)
 {
@@ -160,21 +180,17 @@ gedit_get_command_line_data (void)
 
 		file_list = g_slist_reverse (file_list);
 	}
-
-	if (encoding_charset)
+	
+	if (encoding_charset &&
+	    (gedit_encoding_get_from_charset (encoding_charset) == NULL))
 	{
-		encoding = gedit_encoding_get_from_charset (encoding_charset);
-		if (encoding == NULL)
-			g_print (_("%s: invalid encoding.\n"),
-				 encoding_charset);
-
-		g_free (encoding_charset);
-		encoding_charset = NULL;
+		g_print (_("%s: invalid encoding.\n"),
+			 encoding_charset);
 	}
 }
 
 static guint32
-get_startup_timestamp ()
+get_startup_timestamp (void)
 {
 	const gchar *startup_id_env;
 	gchar *startup_id = NULL;
@@ -238,6 +254,7 @@ static void
 on_message_received (const char *message,
 		     gpointer    data)
 {
+	const GeditEncoding *encoding = NULL;
 	gchar **commands;
 	gchar **params;
 	gint workspace;
@@ -297,7 +314,8 @@ on_message_received (const char *message,
 			gchar **uris;
 
 			line_position = atoi (params[1]);
-			if (params[2] != '\0');
+
+			if (params[2] != '\0')
 				encoding = gedit_encoding_get_from_charset (params[2]);
 
 			n_uris = atoi (params[3]);
@@ -383,18 +401,10 @@ on_message_received (const char *message,
 
 	gtk_window_present (GTK_WINDOW (window));
 
-	/* free the file list and reset to default */
-	g_slist_foreach (file_list, (GFunc) g_object_unref, NULL);
-	g_slist_free (file_list);
-	file_list = NULL;
-
  out:
 	g_strfreev (commands);
-
-	new_window_option = FALSE;
-	new_document_option = FALSE;
-	encoding = NULL;
-	line_position = 0;
+	
+	free_command_line_data ();
 }
 
 /* clientside */
@@ -563,6 +573,29 @@ main (int argc, char *argv[])
 
 #ifdef G_OS_WIN32
 	setup_path ();
+
+	/* If we open gedit from a console get the stdout printing */
+	if (fileno (stdout) != -1 &&
+		_get_osfhandle (fileno (stdout)) != -1)
+	{
+		/* stdout is fine, presumably redirected to a file or pipe */
+	}
+	else
+	{
+		typedef BOOL (* WINAPI AttachConsole_t) (DWORD);
+
+		AttachConsole_t p_AttachConsole =
+			(AttachConsole_t) GetProcAddress (GetModuleHandle ("kernel32.dll"),
+							  "AttachConsole");
+
+		if (p_AttachConsole != NULL && p_AttachConsole (ATTACH_PARENT_PROCESS))
+		{
+			freopen ("CONOUT$", "w", stdout);
+			dup2 (fileno (stdout), 1);
+			freopen ("CONOUT$", "w", stderr);
+			dup2 (fileno (stderr), 2);
+		}
+	}
 #endif
 
 	gtk_init (&argc, &argv);
@@ -591,7 +624,9 @@ main (int argc, char *argv[])
 			gedit_get_command_line_data ();
 
 			send_bacon_message ();
-
+			
+			free_command_line_data ();
+			
 			/* we never popup a window... tell startup-notification
 			 * that we are done.
 			 */
@@ -667,6 +702,11 @@ main (int argc, char *argv[])
 
 		if (file_list != NULL)
 		{
+			const GeditEncoding *encoding = NULL;
+		
+			if (encoding_charset)
+				encoding = gedit_encoding_get_from_charset (encoding_charset);
+		
 			gedit_debug_message (DEBUG_APP, "Load files");
 			_gedit_cmd_load_files_from_prompt (window, 
 							   file_list, 
@@ -682,16 +722,7 @@ main (int argc, char *argv[])
 		gedit_debug_message (DEBUG_APP, "Show window");
 		gtk_widget_show (GTK_WIDGET (window));
 
-		g_slist_foreach (file_list, (GFunc) g_object_unref, NULL);
-		g_slist_free (file_list);
-		file_list = NULL;
-
-		g_free (encoding_charset);
-
-		new_window_option = FALSE;
-		new_document_option = FALSE;
-		encoding = NULL;
-		line_position = 0;
+		free_command_line_data ();
 	}
 
 	gedit_debug_message (DEBUG_APP, "Start gtk-main");		
