@@ -37,6 +37,7 @@
 #include "gedit-file-browser-utils.h"
 #include "gedit-file-browser-error.h"
 #include "gedit-file-browser-widget.h"
+#include "gedit-file-browser-messages.h"
 
 #define WINDOW_DATA_KEY	        	"GeditFileBrowserPluginWindowData"
 #define FILE_BROWSER_BASE_KEY 		"/apps/gedit-2/plugins/filebrowser"
@@ -557,13 +558,8 @@ on_selection_changed_cb (GtkTreeSelection *selection,
 	if (!GEDIT_IS_FILE_BROWSER_STORE (model))
 		return;
 	
-	if (gedit_file_browser_widget_get_num_selected_files_or_directories (data->tree_widget) > 1)
-		sensitive = FALSE;
-	else if (!gedit_file_browser_widget_get_selected_directory (data->tree_widget, &iter))
-		sensitive = FALSE;
-	else
-		sensitive = TRUE;
-	
+	sensitive = gedit_file_browser_widget_get_selected_directory (data->tree_widget, &iter);
+
 	if (sensitive) {
 		gtk_tree_model_get (model, &iter, 
 				    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, 
@@ -579,16 +575,21 @@ on_selection_changed_cb (GtkTreeSelection *selection,
 		sensitive);
 }
 
-#define POPUP_UI ""                          \
-"<ui>"                                       \
-"  <popup name=\"Popup\">"                   \
-"    <placeholder name=\"Popup_Opt1\">"      \
-"      <menuitem action=\"SetActiveRoot\"/>" \
-"    </placeholder>"                         \
-"    <placeholder name=\"Popup_Opt4\">"      \
-"      <menuitem action=\"OpenTerminal\"/>"  \
-"    </placeholder>"                         \
-"  </popup>"                                 \
+#define POPUP_UI ""                             \
+"<ui>"                                          \
+"  <popup name=\"FilePopup\">"                  \
+"    <placeholder name=\"FilePopup_Opt1\">"     \
+"      <menuitem action=\"SetActiveRoot\"/>"    \
+"    </placeholder>"                            \
+"    <placeholder name=\"FilePopup_Opt4\">"     \
+"      <menuitem action=\"OpenTerminal\"/>"     \
+"    </placeholder>"                            \
+"  </popup>"                                    \
+"  <popup name=\"BookmarkPopup\">"              \
+"    <placeholder name=\"BookmarkPopup_Opt1\">" \
+"      <menuitem action=\"SetActiveRoot\"/>"    \
+"    </placeholder>"                            \
+"  </popup>"                                    \
 "</ui>"
 
 static GtkActionEntry extra_actions[] = 
@@ -600,7 +601,7 @@ static GtkActionEntry extra_actions[] =
 };
 
 static GtkActionEntry extra_single_selection_actions[] = {
-	{"OpenTerminal", GTK_STOCK_EXECUTE, N_("_Open terminal here"),
+	{"OpenTerminal", "utilities-terminal", N_("_Open terminal here"),
 	 NULL,
 	 N_("Open a terminal at the currently opened directory"),
 	 G_CALLBACK (on_action_open_terminal)}
@@ -687,11 +688,13 @@ impl_activate (GeditPlugin * plugin, GeditWindow * window)
 	GtkWidget * image;
 	GdkPixbuf * pixbuf;
 	GeditFileBrowserStore * store;
+	gchar *data_dir;
 
-	data = g_new (GeditFileBrowserPluginData, 1);
-	data->auto_root = FALSE;
-	data->end_loading_handle = 0;
-	data->tree_widget = GEDIT_FILE_BROWSER_WIDGET (gedit_file_browser_widget_new ());
+	data = g_new0 (GeditFileBrowserPluginData, 1);
+	
+	data_dir = gedit_plugin_get_data_dir (plugin);
+	data->tree_widget = GEDIT_FILE_BROWSER_WIDGET (gedit_file_browser_widget_new (data_dir));
+	g_free (data_dir);
 
 	g_signal_connect (data->tree_widget,
 			  "uri-activated",
@@ -775,7 +778,10 @@ impl_activate (GeditPlugin * plugin, GeditWindow * window)
 	                  "tab-added",
 	                  G_CALLBACK (on_tab_added_cb),
 	                  data);
-	                  
+	
+	/* Register messages on the bus */
+	gedit_file_browser_messages_register (window, data->tree_widget);
+
 	impl_updateui (plugin, window);
 }
 
@@ -788,6 +794,9 @@ impl_deactivate (GeditPlugin * plugin, GeditWindow * window)
 
 	data = get_plugin_data (window);
 
+	/* Unregister messages from the bus */
+	gedit_file_browser_messages_unregister (window);
+
 	/* Disconnect signals */
 	g_signal_handlers_disconnect_by_func (window, 
 	                                      G_CALLBACK (on_tab_added_cb),
