@@ -538,16 +538,14 @@ gedit_file_browser_store_get_n_columns (GtkTreeModel * tree_model)
 }
 
 static GType
-gedit_file_browser_store_get_column_type (GtkTreeModel * tree_model,
-					  gint index)
+gedit_file_browser_store_get_column_type (GtkTreeModel * tree_model, gint idx)
 {
 	g_return_val_if_fail (GEDIT_IS_FILE_BROWSER_STORE (tree_model),
 			      G_TYPE_INVALID);
-	g_return_val_if_fail (index < GEDIT_FILE_BROWSER_STORE_COLUMN_NUM
-			      && index >= 0, G_TYPE_INVALID);
+	g_return_val_if_fail (idx < GEDIT_FILE_BROWSER_STORE_COLUMN_NUM &&
+			      idx >= 0, G_TYPE_INVALID);
 
-	return GEDIT_FILE_BROWSER_STORE (tree_model)->priv->
-	    column_types[index];
+	return GEDIT_FILE_BROWSER_STORE (tree_model)->priv->column_types[idx];
 }
 
 static gboolean
@@ -697,6 +695,8 @@ gedit_file_browser_store_get_value (GtkTreeModel * tree_model,
 	case GEDIT_FILE_BROWSER_STORE_COLUMN_EMBLEM:
 		g_value_set_object (value, node->emblem);
 		break;
+	default:
+		g_return_if_reached ();
 	}
 }
 
@@ -1288,7 +1288,7 @@ file_browser_node_init (FileBrowserNode * node, GFile * file,
 static FileBrowserNode *
 file_browser_node_new (GFile * file, FileBrowserNode * parent)
 {
-	FileBrowserNode *node = g_new0 (FileBrowserNode, 1);
+	FileBrowserNode *node = g_slice_new0 (FileBrowserNode);
 
 	file_browser_node_init (node, file, parent);
 	return node;
@@ -1299,7 +1299,7 @@ file_browser_node_dir_new (GeditFileBrowserStore * model,
 			   GFile * file, FileBrowserNode * parent)
 {
 	FileBrowserNode *node =
-	    (FileBrowserNode *) g_new0 (FileBrowserNodeDir, 1);
+	    (FileBrowserNode *) g_slice_new0 (FileBrowserNodeDir);
 
 	file_browser_node_init (node, file, parent);
 
@@ -1338,13 +1338,15 @@ static void
 file_browser_node_free (GeditFileBrowserStore * model,
 			FileBrowserNode * node)
 {
-	FileBrowserNodeDir * dir;
-	gchar * uri;
+	gchar *uri;
 
 	if (node == NULL)
 		return;
 
-	if (NODE_IS_DIR (node)) {
+	if (NODE_IS_DIR (node))
+	{
+		FileBrowserNodeDir *dir;
+
 		dir = FILE_BROWSER_NODE_DIR (node);
 
 		if (dir->cancellable) {
@@ -1381,7 +1383,11 @@ file_browser_node_free (GeditFileBrowserStore * model,
 		g_object_unref (node->emblem);
 
 	g_free (node->name);
-	g_free (node);
+	
+	if (NODE_IS_DIR (node))
+		g_slice_free (FileBrowserNodeDir, (FileBrowserNodeDir *)node);
+	else
+		g_slice_free (FileBrowserNode, (FileBrowserNode *)node);
 }
 
 /**
@@ -2347,6 +2353,7 @@ model_load_directory (GeditFileBrowserStore * model,
 		      FileBrowserNode * node)
 {
 	FileBrowserNodeDir *dir;
+	AsyncNode *async;
 
 	g_return_if_fail (NODE_IS_DIR (node));
 
@@ -2365,7 +2372,7 @@ model_load_directory (GeditFileBrowserStore * model,
 	
 	dir->cancellable = g_cancellable_new ();
 	
-	AsyncNode *async = g_new (AsyncNode, 1);
+	async = g_new (AsyncNode, 1);
 	async->dir = dir;
 	async->cancellable = g_object_ref (dir->cancellable);
 	async->original_children = g_slist_copy (dir->children);
@@ -2535,12 +2542,15 @@ set_virtual_root_from_node (GeditFileBrowserStore * model,
 
 	/* Now finally, set the virtual root, and load it up! */
 	model->priv->virtual_root = node;
+
+	/* Notify that the virtual-root has changed before loading up new nodes so that the
+	   "root_changed" signal can be emitted before any "inserted" signals */
+	g_object_notify (G_OBJECT (model), "virtual-root");
+
 	model_fill (model, NULL, &empty);
 
 	if (!NODE_LOADED (node))
 		model_load_directory (model, node);
-
-	g_object_notify (G_OBJECT (model), "virtual-root");
 }
 
 static void
@@ -3080,20 +3090,22 @@ gchar *
 gedit_file_browser_store_get_root (GeditFileBrowserStore * model)
 {
 	g_return_val_if_fail (GEDIT_IS_FILE_BROWSER_STORE (model), NULL);
-	g_return_val_if_fail (model->priv->root != NULL, NULL);
-	g_return_val_if_fail (model->priv->root->file != NULL, NULL);
-
-	return g_file_get_uri (model->priv->root->file);
+	
+	if (model->priv->root == NULL || model->priv->root->file == NULL)
+		return NULL;
+	else
+		return g_file_get_uri (model->priv->root->file);
 }
 
 gchar * 
 gedit_file_browser_store_get_virtual_root (GeditFileBrowserStore * model)
 {
 	g_return_val_if_fail (GEDIT_IS_FILE_BROWSER_STORE (model), NULL);
-	g_return_val_if_fail (model->priv->virtual_root != NULL, NULL);
-	g_return_val_if_fail (model->priv->virtual_root->file != NULL, NULL);
 	
-	return g_file_get_uri (model->priv->virtual_root->file);
+	if (model->priv->virtual_root == NULL || model->priv->virtual_root->file == NULL)
+		return NULL;
+	else
+		return g_file_get_uri (model->priv->virtual_root->file);
 }
 
 void
